@@ -6,6 +6,9 @@ magrittr::`%>%`
 ##' @export
 magrittr::`%<>%`
 
+##' @import tidytree
+NULL
+
 ##' @importFrom tidytree get.fields
 ##' @export
 tidytree::get.fields
@@ -13,11 +16,11 @@ tidytree::get.fields
 ##' @importFrom tidytree get.data
 ##' @export
 tidytree::get.data
-
+ 
 ##' @importFrom tidytree as.treedata
 ##' @export
 tidytree::as.treedata
-
+ 
 ##' @importFrom tidytree ancestor
 ##' @export
 tidytree::ancestor
@@ -56,7 +59,30 @@ tidytree::MRCA
 dplyr::full_join
 
 
+##' @importFrom dplyr inner_join
+##' @export
+dplyr::inner_join
+
 ##' @importFrom tibble as_tibble
 ##' @export
 tibble::as_tibble
 
+##' @importFrom tibble tibble
+##' @export
+tibble::tibble
+
+##' @importFrom tidytree as.phylo
+##' @export
+tidytree::as.phylo
+
+##' @importFrom rlang .data
+##' @export
+rlang::.data
+
+##' @importFrom tidytree drop.tip
+##' @export
+tidytree::drop.tip
+
+##' @importFrom tidytree isTip
+##' @export
+tidytree::isTip
